import textwrap
import click
from pyrecipes.chapter import Chapter
from pyrecipes.cookbook import cookbook
from pyrecipes.errors import ChapterNotFoundError
from pyrecipes.utils import text_border
from colorama import Fore


def render_chapter(chapter: Chapter, describe: bool = False):
    click.echo(text_border(str(chapter), side_symbol=" "))
    for _, recipe in chapter:
        click.echo(recipe)
        if describe:
            click.echo(
                Fore.YELLOW
                + textwrap.fill(
                    recipe.get_docstring(),
                    70,
                    initial_indent="   ",
                    subsequent_indent="   ",
                )
                + Fore.RESET
                + "\n"
            )
    click.echo("")


@click.command()
@click.option(
    "-d",
    "--describe",
    is_flag=True,
    help="Shows descriptions of the recipes",
)
@click.argument("chapter", required=False, default=None, type=int)
def ls(chapter, describe):
    """List recipes

    \b
    The default behaviour lists the titles of all recipes for all chapters.
    Limit the output to a specific recipe by adding an additional 'CHAPTER'
    argument e.g. to list chapter 1 recipes

    \b
        recipes ls 1

    You can also add a '-d' flag to display a short description of each recipe.
    """
    if chapter:
        try:
            render_chapter(cookbook[chapter], describe)
        except ChapterNotFoundError as exc:
            click.echo(exc)
    else:
        for _, chapter in cookbook:
            render_chapter(chapter, describe)



import re


def clean_text(text):
    num, text = text.split("_", maxsplit=1)
    return f"{int(num)}) {text.replace('_', ' ').capitalize()}"


def extract_leading_numbers(text):
    m = re.match(r"^\d+", text)
    if m:
        return int(m.group())


def text_border(text, symbol="=", side_symbol="=", padding=1):
    width = 2 * padding + len(text) + 2
    top = bottom = f"{symbol * width}"
    pad = " " * padding
    return f"{top}\n{side_symbol}{pad}{text}{pad}{side_symbol}\n{bottom}"



import sys
from typing import Mapping
import click
import re
from colorama import Fore, init
from colorama.ansi import AnsiFore
from pyrecipes.chapter import Chapter
from pyrecipes.cookbook import cookbook
from pyrecipes.recipe import SearchMatch
from pyrecipes.utils import text_border

init(autoreset=True)

COLORS = {
    "black": Fore.BLACK,
    "red": Fore.RED,
    "green": Fore.GREEN,
    "yellow": Fore.YELLOW,
    "blue": Fore.BLUE,
    "magenta": Fore.MAGENTA,
    "cyan": Fore.CYAN,
    "white": Fore.WHITE,
    "none": Fore.RESET,
}


def get_n_matches(matches):
    count = 0
    for recipes in matches.values():
        for matches in recipes.values():
            count += sum(match.count for match in matches)
    return count


def render_match(
    pattern: str, match: SearchMatch, color: AnsiFore, ignore_case: bool = False
):
    flags = re.IGNORECASE if ignore_case else 0
    text = re.sub(
        pattern,
        lambda match: f"{color}{match.group()}{Fore.RESET}",
        match.line_text,
        flags=flags,
    ).strip()
    click.echo(f"  Line {match.line_number}: {text}")


def render_matches(
    pattern: str,
    match_dict: Mapping[Chapter, Mapping],
    color: AnsiFore,
    ignore_case: bool = False,
):
    for chapter, recipes in match_dict.items():
        click.echo(text_border(str(chapter)))
        for recipe, matches in recipes.items():
            click.echo(f"{chapter.number}.{recipe}")
            for match in matches:
                render_match(pattern, match, color, ignore_case)
            click.echo("")


@click.command()
@click.argument("pattern", type=str)
@click.option(
    "--color",
    type=click.Choice(COLORS.keys(), case_sensitive=False),
    default="red",
    help="Change the color used to highlight matches. The default is red.",
)
@click.option(
    "-i",
    "--ignore-case",
    is_flag=True,
    default=False,
    help="Make the search case-insensitive",
)
@click.option(
    "-c", "--count-only", is_flag=True, help="Return the count of matches only."
)
def search(pattern, color, ignore_case, count_only):
    """Search the recipes for a pattern

    \b
    - RegEx patterns are supported.
    - Searches are case-sensitive by default. Use the `-i` flag to make searches case-insensitive.
    - Use the `-c` flag to display count of matches.

    """
    color = COLORS.get(color)
    match_dict = cookbook.search(pattern, ignore_case)

    click.echo(f"Found {get_n_matches(match_dict):,} matches")

    if count_only:
        sys.exit(0)

    if match_dict:
        render_matches(pattern, match_dict, color, ignore_case)
